/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

//*****************************************************************************
//* CEC W6100-EVB DRIVER
//* WRITTEN BY FRED EADY
//* VERSION 1.00 A
//* COMPILED USING KEIL MDK v5.28.0.0
//* LAST UPDATE 10/06/2019
//* NOTES:
//*****************************************************************************

//*****************************************************************************
//* INCLUDES
//*****************************************************************************
#include "main.h"
#include "stm32f1xx_hal.h"
#include "wizchip_conf.h"
#include "socket.h"
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include "dhcpv6.h"
//*****************************************************************************
//* FUNCTION PROTOTYPES
//*****************************************************************************
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_SPI2_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_USART2_UART_Init(void);
void wiz_csLO(void);
void wiz_csHI(void);
uint8_t spi_rb(void);
void spi_wb(uint8_t b);
uint8_t CharInRing(void);
uint8_t readring(void);
void showNetInfo(void);
void init(void);
void print_network_information();	
//*****************************************************************************
//* TYPEDEFS
//*****************************************************************************
SPI_HandleTypeDef hspi2;

UART_HandleTypeDef huart1;
UART_HandleTypeDef huart2;

typedef struct{
	uint16_t	fentry:1;
	uint16_t	fmonitor:1;
}FFLAGS;
FFLAGS flags;

enum{
	MODE_DHCP,
	MODE_MONITOR,
	MODE_CONNECT,
	MODE_QUIT
};
uint8_t syslock = SYS_NET_LOCK;
wiz_NetInfo netInfo = { .mac = {
								0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED
								},
							.ip = {
								192, 168, 10, 20
							},
							.sn = {
								255, 255, 255, 0
							},
							.gw = {
								192, 168, 10, 1
							},
							.dns = {
								192, 168, 10, 1
							},
							.lla={
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0
							},
							.gua={
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0
							},
							.sn6={
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0
							},
							.gw6={
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0,
								0, 0, 0, 0
							}
};
//*****************************************************************************
//* CCS MACROS
//*****************************************************************************
#define make8(var,offset)	((uint16_t)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((uint16_t)varhigh & 0xFF)* 0x100) + ((uint32_t)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) 								\
				((uint32_t)var1<<24)+((uint32_t)var2<<16)+ 	\
				((uint32_t)var3<<8)+((uint32_t)var4)
//*****************************************************************************
//* PRINTF
//*****************************************************************************
int fputc(int ch, FILE *f)
	{
			HAL_UART_Transmit(&huart1, (uint8_t *)&ch, 1, 0xFFFF);
			return ch;
	}
//*****************************************************************************
//* USART RECEIVE BUFFERS SETUP
//* 1,2,4,8,16,32,64,128 or 256 uint8_tS 
//*****************************************************************************
#define USART1_RX_BUFFER_SIZE  256      
#define USART1_RX_BUFFER_MASK ( USART1_RX_BUFFER_SIZE - 1 )
uint8_t USART1_RxBuf[USART1_RX_BUFFER_SIZE];
uint8_t USART1_RxTail;
uint8_t USART1_RxHead;

//#define USART2_RX_BUFFER_SIZE  256      
//#define USART2_RX_BUFFER_MASK ( USART2_RX_BUFFER_SIZE - 1 )
//uint8_t USART2_RxBuf[USART2_RX_BUFFER_SIZE];
//uint8_t USART2_RxTail;
//uint8_t USART2_RxHead;
//*****************************************************************************
//* USER VARIABLES
//*****************************************************************************
uint8_t		bufIndx;
uint8_t		pstate;
uint8_t		lastpstate;
uint8_t		scratch8;
uint16_t	scratch16;
//*****************************************************************************
//* SOCKET USER VARIABLES
//*****************************************************************************
#define 	DATA_BUF_SIZE 2048
uint8_t 	sktRxBuf[2048];
uint8_t 	sktTxBuf[64];
uint16_t 	sktIndx;
uint8_t 	retVal;
uint8_t 	sockStatus;
int16_t 	rcvLen;
uint8_t 	rcvBuf[20];
uint8_t 	bufSize[] = {2, 2, 2, 2};
uint8_t 	rxBuf[128];
uint8_t 	dhcpBuf[128];
uint8_t 	spiderIP[] = {0xD0,0x50,0x26,0xFE};//208.80.38.254
//*****************************************************************************
//* WIZNET SPI CS FUNCTIONS
//*****************************************************************************
void wiz_csLO(void)
{
	HAL_GPIO_WritePin(SPI2_NSS_GPIO_Port, SPI2_NSS_Pin, GPIO_PIN_RESET);
}
void wiz_csHI(void)
{
	HAL_GPIO_WritePin(SPI2_NSS_GPIO_Port, SPI2_NSS_Pin, GPIO_PIN_SET);
}
//*****************************************************************************
//* SPI READ/WRITE FUNCTIONS
//*****************************************************************************
uint8_t spi_rb(void) 
{
	uint8_t rbuf;
	HAL_SPI_Receive(&hspi2, &rbuf, 1, 0xFFFFFFFF);
	return rbuf;
}

void spi_wb(uint8_t b) 
{
	HAL_SPI_Transmit(&hspi2, &b, 1, 0xFFFFFFFF);
}
//*****************************************************************************
//*   CHECK FOR CHARACTER IN RING
//*****************************************************************************
uint8_t CharInRing(void)
{
  return(USART1_RxHead != USART1_RxTail);
}
//*****************************************************************************
//*   GET BYTE FROM RX RING BUFFER
//*****************************************************************************
uint8_t readring(void)
{
  __HAL_UART_DISABLE_IT(&huart1,UART_IT_RXNE);
  uint8_t tmptail;
  // wait for incomming data  
  while ( USART1_RxHead == USART1_RxTail );
  // calculate buffer index  
  tmptail = ( USART1_RxTail + 1 ) & USART1_RX_BUFFER_MASK;
	// store new index
  USART1_RxTail = tmptail;          
    __HAL_UART_ENABLE_IT(&huart1,UART_IT_RXNE);
	// return data
  return USART1_RxBuf[tmptail];     
}
//*****************************************************************************
//* SHOW NETWORK CONFIGURATION
//*****************************************************************************
void showNetInfo(void)
{
  printf("Network configuration:\r\n");	
  printf("  IP ADDRESS:  %d.%d.%d.%d\r\n", netInfo.ip[0],netInfo.ip[1], netInfo.ip[2], netInfo.ip[3]);									
  printf("  MAC ADDRESS: 0x%02X:0x%02X:0x%02X:0x%02X:0x%02X:0x%02X\r\n", netInfo.mac[0],netInfo.mac[1], netInfo.mac[2], netInfo.mac[3], netInfo.mac[4], netInfo.mac[5]);
  printf("  NETMASK:     %d.%d.%d.%d\r\n", netInfo.sn[0], netInfo.sn[1], netInfo.sn[2], netInfo.sn[3]);								
  printf("  GATEWAY:     %d.%d.%d.%d\r\n", netInfo.gw[0], netInfo.gw[1], netInfo.gw[2], netInfo.gw[3]);										
}
void print_network_information(void)
{
    uint8_t tmp_array[16];
    uint8_t i;
	wizchip_getnetinfo(&netInfo);
	printf("Mac address: %02x:%02x:%02x:%02x:%02x:%02x\r\n",netInfo.mac[0],netInfo.mac[1],netInfo.mac[2],netInfo.mac[3],netInfo.mac[4],netInfo.mac[5]);
	printf("IP address : %d.%d.%d.%d\n\r",netInfo.ip[0],netInfo.ip[1],netInfo.ip[2],netInfo.ip[3]);
	printf("SM Mask	   : %d.%d.%d.%d\n\r",netInfo.sn[0],netInfo.sn[1],netInfo.sn[2],netInfo.sn[3]);
	printf("Gate way   : %d.%d.%d.%d\n\r",netInfo.gw[0],netInfo.gw[1],netInfo.gw[2],netInfo.gw[3]);
	printf("DNS Server : %d.%d.%d.%d\n\r",netInfo.dns[0],netInfo.dns[1],netInfo.dns[2],netInfo.dns[3]);
	getGA6R(tmp_array);
    printf("GW6 : %04X:%04X", ((uint16_t)tmp_array[0] << 8) | ((uint16_t)tmp_array[1]),
    		((uint16_t)tmp_array[2] << 8) | ((uint16_t)tmp_array[3]));
    printf(":%04X:%04X", ((uint16_t)tmp_array[4] << 8) | ((uint16_t)tmp_array[5]),
    		((uint16_t)tmp_array[6] << 8) | ((uint16_t)tmp_array[7]));
    printf(":%04X:%04X", ((uint16_t)tmp_array[8] << 8) | ((uint16_t)tmp_array[9]),
    		((uint16_t)tmp_array[10] << 8) | ((uint16_t)tmp_array[11]));
    printf(":%04X:%04X\r\n", ((uint16_t)tmp_array[12] << 8) | ((uint16_t)tmp_array[13]),
    		((uint16_t)tmp_array[14] << 8) | ((uint16_t)tmp_array[15]));

	getLLAR(tmp_array);
	printf("LLA : %04X:%04X", ((uint16_t)tmp_array[0] << 8) | ((uint16_t)tmp_array[1]),
			((uint16_t)tmp_array[2] << 8) | ((uint16_t)tmp_array[3]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[4] << 8) | ((uint16_t)tmp_array[5]),
			((uint16_t)tmp_array[6] << 8) | ((uint16_t)tmp_array[7]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[8] << 8) | ((uint16_t)tmp_array[9]),
			((uint16_t)tmp_array[10] << 8) | ((uint16_t)tmp_array[11]));
	printf(":%04X:%04X\r\n", ((uint16_t)tmp_array[12] << 8) | ((uint16_t)tmp_array[13]),
			((uint16_t)tmp_array[14] << 8) | ((uint16_t)tmp_array[15]));
	getGUAR(tmp_array);
	printf("GUA : %04X:%04X", ((uint16_t)tmp_array[0] << 8) | ((uint16_t)tmp_array[1]),
			((uint16_t)tmp_array[2] << 8) | ((uint16_t)tmp_array[3]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[4] << 8) | ((uint16_t)tmp_array[5]),
			((uint16_t)tmp_array[6] << 8) | ((uint16_t)tmp_array[7]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[8] << 8) | ((uint16_t)tmp_array[9]),
			((uint16_t)tmp_array[10] << 8) | ((uint16_t)tmp_array[11]));
	printf(":%04X:%04X\r\n", ((uint16_t)tmp_array[12] << 8) | ((uint16_t)tmp_array[13]),
			((uint16_t)tmp_array[14] << 8) | ((uint16_t)tmp_array[15]));

	getSUB6R(tmp_array);
	printf("SUB6 : %04X:%04X", ((uint16_t)tmp_array[0] << 8) | ((uint16_t)tmp_array[1]),
			((uint16_t)tmp_array[2] << 8) | ((uint16_t)tmp_array[3]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[4] << 8) | ((uint16_t)tmp_array[5]),
			((uint16_t)tmp_array[6] << 8) | ((uint16_t)tmp_array[7]));
	printf(":%04X:%04X", ((uint16_t)tmp_array[8] << 8) | ((uint16_t)tmp_array[9]),
			((uint16_t)tmp_array[10] << 8) | ((uint16_t)tmp_array[11]));
	printf(":%04X:%04X\r\n", ((uint16_t)tmp_array[12] << 8) | ((uint16_t)tmp_array[13]),
			((uint16_t)tmp_array[14] << 8) | ((uint16_t)tmp_array[15]));


	printf("\r\nNETCFGLOCK : %x\r\n", getNETLCKR());
}
void wizReset(void)
{
	HAL_GPIO_WritePin(RSTn_GPIO_Port, RSTn_Pin, GPIO_PIN_RESET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(RSTn_GPIO_Port, RSTn_Pin, GPIO_PIN_SET);
	HAL_Delay(10);
}
void W6100Register_read(void)
{
	int i;
	printf("                    ----register read----\r\n");
	printf("Address | ");
	for (i = 0; i < 16; i++)
		printf("%02x ", i);
	printf("\r\n---------------------------------------------------------");
	for (i = 0; i < 0x0090; i++)
	{
		if (i % 16 == 0)
			printf("\r\n  %04x  | ", i);
		printf("%02x ", WIZCHIP_READ(_W6100_IO_BASE_ + (i << 8) + (WIZCHIP_CREG_BLOCK << 3)));
	}
	printf("\r\n");
}void initW6100(void)
{
	wizReset();

	uint8_t temp;
	unsigned char W6100_AdrSet[2][8] = {{2, 2, 2, 2, 2, 2, 2, 2}, {2, 2, 2, 2, 2, 2, 2, 2}};
		/*
	do
	{
		if (ctlwizchip(CW_GET_PHYLINK, (void *)&temp) == -1)
		{
			printf("Unknown PHY link status.\r\n");
		}
	} while (temp == PHY_LINK_OFF);
		*/
	printf("PHY OK.\r\n");

	temp = IK_DEST_UNREACH;

	if (ctlwizchip(CW_INIT_WIZCHIP, (void *)W6100_AdrSet) == -1)
	{
		printf("W6100 initialized fail.\r\n");
	}

	if (ctlwizchip(CW_SET_INTRMASK, &temp) == -1)
	{
		printf("W6100 interrupt\r\n");
	}
	//printf("interrupt mask: %02x\r\n",getIMR());
}

void init(void)
{
	USART1_RxHead = 0;
	USART1_RxTail = 0;
	pstate = MODE_DHCP;
}

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */
  

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();
  /* Configure the system clock */
  SystemClock_Config();
  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_SPI2_Init();
  MX_USART1_UART_Init();
  MX_USART2_UART_Init();
	
  __HAL_UART_ENABLE_IT(&huart1,UART_IT_RXNE);

	W6100Register_read();
	while(1);
	initW6100();
	ctlwizchip(CW_SYS_UNLOCK, &syslock);
	reg_wizchip_cs_cbfunc(wiz_csLO, wiz_csHI);
  reg_wizchip_spi_cbfunc(spi_rb, spi_wb,0,0);
	wizchip_init(bufSize, bufSize);
	wizchip_setnetinfo(&netInfo);	
	showNetInfo();
	//print_network_information();	
	do{
	switch(pstate)
	{
//*****************************************************************************
//* MODE_MONITOR
//*****************************************************************************
		case MODE_MONITOR:
			if(flags.fmonitor == 0)
			{
				printf("MODE_MONITOR\r\n");
				flags.fmonitor = 1;
			}
		break;
//*****************************************************************************
//* MODE_DHCP
//*****************************************************************************
		case MODE_DHCP:
			scratch8 = 0;
			DHCP_init(0,dhcpBuf);
			printf("DHCP Init\r\n");
			do{
			switch(DHCP_run2())
			{
				case DHCP_FAILED:  			//< Processing Fail
					printf("DHCP FAILED\r\n");
					lastpstate = pstate;
					pstate = MODE_MONITOR;
				break;
				case DHCP_RUNNING:     	//< Processing DHCP protocol
					if(scratch8 == 0)
					{
						printf("DHCP RUNNING\r\n");
						scratch8++;
					}
				break;
				case DHCP_IP_ASSIGN:   	//< First Occupy IP from DHPC server      (if cbfunc == null, act as default default_ip_assign)
					printf("DHCP IP ASSIGN\r\n");
				break;
				case DHCP_IP_CHANGED:  	//< Change IP address by new ip from DHCP (if cbfunc == null, act as default default_ip_update)
					printf("DHCP IP CHANGED\r\n");
				break;
				case DHCP_IP_LEASED:   	//< Stand by
							printf("DHCP IP LEASED\r\n");
							wizchip_getnetinfo(&netInfo);
							showNetInfo();
							lastpstate = pstate;
							pstate = MODE_CONNECT;
				break;
				case DHCP_STOPPED:   
					printf("DHCP STOPPED\r\n");
					lastpstate = pstate;
					pstate = MODE_MONITOR;
				break;
			}
		}while(pstate == MODE_DHCP);
		break;
//*****************************************************************************
//* MODE_CONNECT
//*****************************************************************************
				case MODE_CONNECT:
					printf("MODE_CONNECT\r\n");
/*				
					retVal = socket(0, Sn_MR_TCP, 5000, 0);
					if(retVal == 0)
					{
						printf("Socket 0 opened.\r\n");
						retVal = connect(0, spiderIP, 587);
						if(retVal == SOCK_OK)
						{
							printf("Connected\r\n");
							HAL_Delay(2000);
							rcvLen = 0;
							do{
								rcvLen = getSn_RX_RSR(0);
							}while(rcvLen == 0);
							recv(0, sktRxBuf, rcvLen); 
							HAL_UART_Transmit(&huart2, sktRxBuf, rcvLen, 0xFFFF);
							
							sktIndx = 0;
							do{
							if(sktRxBuf[sktIndx]   == '2' &&
								 sktRxBuf[sktIndx+1] == '2' &&
								 sktRxBuf[sktIndx+2] == '0' &&
								 sktRxBuf[sktIndx+3] == ' ')
								{
									lastpstate = pstate;
									pstate = MODE_EHLO;
								}
							}while(pstate == MODE_CONNECT || sktIndx > rcvLen);
							if(sktIndx > rcvLen)
							{
								lastpstate = pstate;
								pstate = MODE_MONITOR;
								close(0);
							}
						}
					}
*/
								lastpstate = pstate;
								pstate = MODE_MONITOR;
					break;
  }
}while(1);
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL6;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief SPI2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI2_Init(void)
{

  /* USER CODE BEGIN SPI2_Init 0 */

  /* USER CODE END SPI2_Init 0 */

  /* USER CODE BEGIN SPI2_Init 1 */

  /* USER CODE END SPI2_Init 1 */
  /* SPI2 parameter configuration*/
  hspi2.Instance = SPI2;
  hspi2.Init.Mode = SPI_MODE_MASTER;
  hspi2.Init.Direction = SPI_DIRECTION_2LINES;
  hspi2.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi2.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi2.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi2.Init.NSS = SPI_NSS_SOFT;
  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_2;
  hspi2.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi2.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi2.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi2.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI2_Init 2 */

  /* USER CODE END SPI2_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 115200;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOC_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, ledRED_Pin|ledGRN_Pin|ledBLU_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(SPI2_NSS_GPIO_Port, SPI2_NSS_Pin|RSTn_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : User0_Pin User1_Pin */
  GPIO_InitStruct.Pin = User0_Pin|User1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : INTn_Pin RDn_Pin WRn_Pin */
  GPIO_InitStruct.Pin = INTn_Pin|RDn_Pin|WRn_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

  /*Configure GPIO pins : ledRED_Pin ledGRN_Pin ledBLU_Pin */
  GPIO_InitStruct.Pin = ledRED_Pin|ledGRN_Pin|ledBLU_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : SPI2_NSS_Pin RSTn_Pin */
  GPIO_InitStruct.Pin = SPI2_NSS_Pin|RSTn_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(SPI2_NSS_GPIO_Port, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
